#include "mbed.h"


DigitalOut myled(LED2);     // Pas encore utilisé
AnalogIn   phototo(A1);    // Renvoie un float entre [1.0 ; 0.0]
                   // Light => no resistance
                   // Dark => high resistance

Serial     pc(USBTX, USBRX);

const int changements_par_tour = 3*2;   // HandSpinner avec 3 branches
const int tours_par_mesure = 2;
const int timeout_ms = 4 * 1000;    // 4 secondes
const int TAILLE_RECORD = 1000;

struct Seuil {
    float haut;
    float bas;
};

Timer t;        // https://developer.mbed.org/handbook/Timer
                    // https://developer.mbed.org/questions/61002/Equivalent-to-Arduino-millis/
                    
Seuil calibration() {
    float  v, v_max, v_min, delta;
    Seuil s;
    
    v = phototo.read();
    v_max = v;
    v_min = v;
    
    t.reset();
    t.start();
    for (int i=0; i<200 ; i++) {
            v = phototo.read();         // float entre [1.0 ; 0.0]
            //phototo.read_u16();       // @TODO  renvoie un entier [65535 ; 0]  (plus rapide)
            if      (v > v_max)  v_max = v;
            else if (v < v_min)  v_min = v;
            wait_ms(5); 
    }
    t.stop();
    delta = (v_max - v_min) / 3.0;
    s.haut = v_max - delta;
    s.bas  = v_min + delta;
    pc.printf("Calibration [%.3f ; %.3f] [%.3f; %.3f] sur %d ms.\t", v_max, s.haut, s.bas, v_min, t.read_ms());
    return s;
}


void affiche_mesures(float mesures[], int nb_mesures)  {
    if ( nb_mesures > TAILLE_RECORD )  nb_mesures = TAILLE_RECORD;
    pc.printf("\r\n---------------------------------\r\n");
    for (int i=0; i < nb_mesures ; i++)  pc.printf("%f;", mesures[i]);
    pc.printf("\r\n---------------------------------\r\n");
}
    

int main() {
    pc.baud( 115200 ); // 9600 14400 57600 115200
    //pc.format(8, SerialBase::None, 1);  // Optional (default)
    
    pc.printf("Demarrage ! \r\n");
    
    bool jour_ou_nuit, precedent;
    int changements, mesures;
    int temps_par_tour_ms;
    Seuil seuil;      
    float value, valpre, moy;
    float record[TAILLE_RECORD];
    
    
    while(true) {

        seuil = calibration();
        valpre = seuil.haut;     // initialise la valeur "precedente"
        
        changements = 0; mesures = 0;
        t.reset();                
        t.start();
        while ( ( changements < ( changements_par_tour * tours_par_mesure ) )
                && ( t.read_ms() < timeout_ms ) )  {
            
            value = phototo.read();
            //printf("v %f\r\n", value);  // debug, mais bien trop lent!
            if (mesures < TAILLE_RECORD)  record[mesures] = value;
            mesures++;  // Attention de ne pas depasser!
    
            moy = (value + valpre) / 2.0;       // on lisse 
            valpre = value;
            if      (moy > seuil.haut)  jour_ou_nuit = true;
            else if (moy < seuil.bas)   jour_ou_nuit = false;
            else continue;
            
             if ( jour_ou_nuit != precedent )  {
                precedent = jour_ou_nuit;
                changements = changements + 1;
            }
         }
         t.stop();
         
        pc.printf(" %d mesures et %d changements \t", mesures, changements);
        
        if ( t.read_ms() >= timeout_ms ) {
            pc.printf("Timeout  ... \r\n");
            continue;
        }
            
        temps_par_tour_ms = t.read_ms() / tours_par_mesure;
        if (temps_par_tour_ms != 0)
            pc.printf("Un tour en %d ms ;\t %d tours/seconde \r\n",  temps_par_tour_ms, 1000/temps_par_tour_ms); 
        else
            pc.printf("Etrange ! le temps mesure est nul !?\r\n");
        affiche_mesures(record, mesures);
        wait_ms(100);       
    }
}

     